@@ -28,11 +28,9 @@ class ComputeStatistics(QtWidgets.QDialog):
         self.pushButton_path.clicked.connect(self.on_browse)
         self.comboBox.currentIndexChanged.connect(self.on_combobox)
         # Populate statistics methods
-        self.listWidget_stats.clear()
-        for meth in STAT_METHODS:
-            wid = QtWidgets.QListWidgetItem(meth)
-            wid.setCheckState(2)
-            self.listWidget_stats.addItem(wid)
+        self.bulklist_stats.set_title("Statistical methods")
+        self.bulklist_stats.set_items(STAT_METHODS)
+        self.bulklist_stats.on_select_all()
         # Populate filter ray comboBox
         self.comboBox_filter_ray.clear()
         self.comboBox_filter_ray.addItem("No filtering", None)
@@ -40,12 +38,16 @@ class ComputeStatistics(QtWidgets.QDialog):
             if slot.slot_used:
                 raytext = "Ray {} ({})".format(ii, slot.name)
                 self.comboBox_filter_ray.addItem(raytext, slot.identifier)
+        # initialize feature list
+        self.bulklist_features.set_title("Features")
         # initialize rest
         if len(self.pipeline.slots) == 0:
             self.comboBox.setCurrentIndex(1)
         else:
             self.comboBox.setCurrentIndex(0)
         self.on_combobox()  # computes self.features
+        # Only select innate features
+        self.on_select_features_innate()
 
     def done(self, r):
         if r:
@@ -60,16 +62,9 @@ class ComputeStatistics(QtWidgets.QDialog):
     def export_statistics(self):
         """Export statistics to .tsv"""
         # get features
-        features = []
-        for ii in range(self.listWidget_features.count()):
-            if self.listWidget_features.item(ii).checkState() == 2:
-                features.append(self.features[ii])
+        features = self.bulklist_features.get_selection()
         # get methods
-        methods = []
-        for ii in range(self.listWidget_stats.count()):
-            if self.listWidget_stats.item(ii).checkState() == 2:
-                methods.append(STAT_METHODS[ii])
-
+        methods = self.bulklist_stats.get_selection()
         prog = QtWidgets.QProgressDialog("Computing statistics...", "Abort", 1,
                                          1, self)
         prog.setMinimumDuration(0)
@@ -149,9 +144,9 @@ class ComputeStatistics(QtWidgets.QDialog):
         # Write rest
         with opath.open("a", encoding="utf-8") as fd:
             for line in header:
-                fd.write("# " + line + "\r\n")
+                fd.write("# " + line + "\n")
             for line in data:
-                fd.write(line + "\r\n")
+                fd.write(line + "\n")
         return True  # True means success
 
     def on_browse(self):
@@ -179,6 +174,22 @@ class ComputeStatistics(QtWidgets.QDialog):
             self.widget_other.hide()
             self.update_feature_list(use_pipeline=True)
 
+    def on_select_features_innate(self):
+        """Only select all innate features of the first dataset"""
+
+        if self.pipeline.num_slots:
+            ds = self.pipeline.get_dataset(0)
+            features_innate = ds.features_innate
+            lw = self.bulklist_features.listWidget
+            for ii in range(lw.count()):
+                wid = lw.item(ii)
+                for feat in features_innate:
+                    if wid.data(101) == feat:
+                        wid.setCheckState(QtCore.Qt.CheckState.Checked)
+                        break
+                else:
+                    wid.setCheckState(QtCore.Qt.CheckState.Unchecked)
+
     def update_feature_list(self, use_pipeline=True):
         if use_pipeline:
             self.features = self.pipeline.get_features(scalar=True,
@@ -195,8 +206,5 @@ class ComputeStatistics(QtWidgets.QDialog):
             lf = sorted(zip(labs, features))
             self.features = [it[1] for it in lf]
 
-        self.listWidget_features.clear()
-        for feat in self.features:
-            wid = QtWidgets.QListWidgetItem(dclab.dfn.get_feature_label(feat))
-            wid.setCheckState(0)
-            self.listWidget_features.addItem(wid)
+        labels = [dclab.dfn.get_feature_label(feat) for feat in self.features]
+        self.bulklist_features.set_items(self.features, labels)
