@@ -8,7 +8,7 @@ import pyqtgraph as pg
 from scipy.ndimage import binary_erosion
 
 from ..compute.comp_stats import STAT_METHODS
-from .. import idiom
+from ... import idiom
 from ..widgets import show_wait_cursor
 
 
@@ -38,6 +38,11 @@ class QuickView(QtWidgets.QWidget):
             cb.addItem("linear", "linear")
             cb.addItem("logarithmic", "log")
 
+        # Set marker hue options (with data)
+        self.comboBox_hue.clear()
+        self.comboBox_hue.addItem("KDE", "kde")
+        self.comboBox_hue.addItem("feature", "feature")
+
         # settings button
         self.toolButton_event.toggled.connect(self.on_tool)
         self.toolButton_poly.toggled.connect(self.on_tool)
@@ -79,6 +84,8 @@ class QuickView(QtWidgets.QWidget):
         self.tabWidget_event.currentChanged.connect(
             self.on_event_scatter_update)
 
+        # apply button
+        self.toolButton_apply.clicked.connect(self.plot)
         # value changed signals for plot
         self.signal_widgets = [self.checkBox_downsample,
                                self.spinBox_downsample,
@@ -87,14 +94,17 @@ class QuickView(QtWidgets.QWidget):
                                self.comboBox_xscale,
                                self.comboBox_yscale,
                                self.checkBox_isoelastics,
+                               self.comboBox_z_hue,
+                               self.comboBox_hue,
+                               self.checkBox_hue,
                                ]
         for w in self.signal_widgets:
             if hasattr(w, "currentIndexChanged"):
-                w.currentIndexChanged.connect(self.plot)
+                w.currentIndexChanged.connect(self.plot_auto)
             elif hasattr(w, "stateChanged"):
-                w.stateChanged.connect(self.plot)
+                w.stateChanged.connect(self.plot_auto)
             elif hasattr(w, "valueChanged"):
-                w.valueChanged.connect(self.plot)
+                w.valueChanged.connect(self.plot_auto)
         # copy statistics to clipboard
         self.toolButton_stats2clipboard.clicked.connect(
             self.on_stats2clipboard)
@@ -136,6 +146,9 @@ class QuickView(QtWidgets.QWidget):
             "scale x": self.comboBox_xscale.currentData(),
             "scale y": self.comboBox_yscale.currentData(),
             "isoelastics": self.checkBox_isoelastics.isChecked(),
+            "marker hue": self.checkBox_hue.isChecked(),
+            "marker hue value": self.comboBox_hue.currentData(),
+            "marker hue feature": self.comboBox_z_hue.currentData(),
         }
         event = {
             "index": self.spinBox_event.value(),
@@ -160,17 +173,23 @@ class QuickView(QtWidgets.QWidget):
         # downsampling
         self.checkBox_downsample.setChecked(plot["downsampling"])
         self.spinBox_downsample.setValue(plot["downsampling value"])
-        # axes labels
+        self.checkBox_hue.setChecked(plot["marker hue"])
+        # combo box key selection
         self.update_feature_choices()
-        idx = self.comboBox_x.findData(plot["axis x"])
-        self.comboBox_x.setCurrentIndex(idx)
-        idy = self.comboBox_y.findData(plot["axis y"])
-        self.comboBox_y.setCurrentIndex(idy)
-        # scaling
-        idxs = self.comboBox_xscale.findData(plot["scale x"])
-        self.comboBox_xscale.setCurrentIndex(idxs)
-        idys = self.comboBox_yscale.findData(plot["scale y"])
-        self.comboBox_yscale.setCurrentIndex(idys)
+        for key, cb in [
+            # axes labels
+            ("axis x", self.comboBox_x),
+            ("axis y", self.comboBox_y),
+            # scaling
+            ("scale x", self.comboBox_xscale),
+            ("scale y", self.comboBox_yscale),
+            # marker hue
+            ("marker hue value", self.comboBox_hue),
+            ("marker hue feature", self.comboBox_z_hue),
+        ]:
+            idx = cb.findData(plot[key])
+            idx = idx if idx > 0 else 0
+            cb.setCurrentIndex(idx)
         # isoelastics
         self.checkBox_isoelastics.setChecked(plot["isoelastics"])
         for tb in self.signal_widgets:
@@ -190,7 +209,7 @@ class QuickView(QtWidgets.QWidget):
     def _check_file_open(self, rtdc_ds):
         """Check whether a dataset is still open"""
         if isinstance(rtdc_ds, dclab.rtdc_dataset.RTDC_HDF5):
-            if rtdc_ds._h5:
+            if rtdc_ds.h5file:
                 # the file is open
                 isopen = True
             else:
@@ -206,10 +225,6 @@ class QuickView(QtWidgets.QWidget):
         # Initially, only show the info about how QuickView works
         self.widget_tool.setEnabled(False)
         self.widget_scatter.hide()
-        # Hide settings/events by default
-        self.widget_event.hide()
-        self.widget_settings.hide()
-        self.widget_poly.hide()
         # show the how-to label
         self.label_howto.show()
         # hide the no events label
@@ -265,7 +280,7 @@ class QuickView(QtWidgets.QWidget):
             if state["event"]["image contour"]:
                 # Compute contour image from mask. If you are wondering
                 # whether this is kosher, please take a look at issue #76:
-                # https://github.com/ZELLMECHANIK-DRESDEN/dclab/issues/76
+                # https://github.com/DC-analysis/dclab/issues/76
                 cont = mask ^ binary_erosion(mask)
                 # set red contour pixel values in original image
                 cellimg[cont, 0] = int(255*.7)
@@ -329,7 +344,11 @@ class QuickView(QtWidgets.QWidget):
             # get corrected index
             event = np.where(plotted)[0][point.index()]
             if "image" in self.rtdc_ds:
-                cellimg, imkw = self.get_event_image(self.rtdc_ds, event)
+                try:
+                    cellimg, imkw = self.get_event_image(self.rtdc_ds, event)
+                except IndexError:
+                    # the plot got updated, and we still have the old data
+                    cellimg, imkw = self.get_event_image(self.rtdc_ds, 0)
                 self.imageView_image_poly.setImage(cellimg, **imkw)
                 self.imageView_image_poly.show()
             else:
@@ -474,20 +493,25 @@ class QuickView(QtWidgets.QWidget):
             pass
         else:
             # keep everything as-is but update the sizes
-            show_event = self.widget_event.isVisible()
-            show_settings = self.widget_settings.isVisible()
-            show_poly = self.widget_poly.isVisible()
+            show_event = self.stackedWidget.currentWidget() is self.page_event
+            show_settings = self.stackedWidget.currentWidget() \
+                is self.page_settings
+            show_poly = self.stackedWidget.currentWidget() is self.page_poly
 
         # toolbutton checked
         self.toolButton_event.setChecked(show_event)
         self.toolButton_poly.setChecked(show_poly)
         self.toolButton_settings.setChecked(show_settings)
 
-        # widget visibility
-        self.widget_event.setVisible(show_event)
+        # stack widget visibility
+        if show_event:
+            self.stackedWidget.setCurrentWidget(self.page_event)
+        elif show_settings:
+            self.stackedWidget.setCurrentWidget(self.page_settings)
+        elif show_poly:
+            self.stackedWidget.setCurrentWidget(self.page_poly)
+
         self.widget_scatter.select.setVisible(show_event)  # point in scatter
-        self.widget_poly.setVisible(show_poly)
-        self.widget_settings.setVisible(show_settings)
 
         if show_event:
             # update event plot (maybe axes changed)
@@ -498,16 +522,8 @@ class QuickView(QtWidgets.QWidget):
 
         if not show_poly:
             self.on_poly_done()
-        # set size
+
         self.update()
-        ws = self.sizeHint()
-        mdiwin = self.parent()
-        geom = mdiwin.geometry()
-        geom.setWidth(ws.width())
-        geom.setHeight(ws.height())
-        mdiwin.setGeometry(geom)
-        mdiwin.adjustSize()
-        mdiwin.update()
 
     @show_wait_cursor
     @QtCore.pyqtSlot()
@@ -516,7 +532,15 @@ class QuickView(QtWidgets.QWidget):
         if self.rtdc_ds is not None:
             plot = self.__getstate__()["plot"]
             downsample = plot["downsampling"] * plot["downsampling value"]
-
+            hue_kwargs = {}
+            if self.checkBox_hue.isChecked():
+                hue_type = self.comboBox_hue.currentData()
+                if hue_type == "kde":
+                    hue_kwargs = {"kde_type": "histogram"}
+                if hue_type == "feature":
+                    hue_kwargs = {"feat": self.comboBox_z_hue.currentData()}
+            else:
+                hue_type = "none"
             self.widget_scatter.plot_data(rtdc_ds=self.rtdc_ds,
                                           slot=self.slot,
                                           downsample=downsample,
@@ -524,6 +548,8 @@ class QuickView(QtWidgets.QWidget):
                                           yax=plot["axis y"],
                                           xscale=plot["scale x"],
                                           yscale=plot["scale y"],
+                                          hue_type=hue_type,
+                                          hue_kwargs=hue_kwargs,
                                           isoelastics=plot["isoelastics"])
             # make sure the correct plot items are visible
             # (e.g. scatter select)
@@ -535,6 +561,23 @@ class QuickView(QtWidgets.QWidget):
                 dclab.dfn.get_feature_label(plot["axis y"]))
             self.show_statistics()
 
+    @QtCore.pyqtSlot()
+    def plot_auto(self):
+        """Update the plot only if the "Auto-apply" checkbox is checked"""
+        if self.checkBox_auto_apply.isChecked():
+            sender = self.sender()
+            for cb, sen in [
+                (self.checkBox_downsample, [self.spinBox_downsample]),
+                (self.checkBox_hue, [self.comboBox_hue,
+                                     self.comboBox_z_hue])]:
+                # Do not replot if the user changes the options for a
+                # disabled settings (e.g. downsampling, hue)
+                if sender in sen:
+                    if not cb.isChecked():
+                        break
+            else:
+                self.plot()
+
     @show_wait_cursor
     @QtCore.pyqtSlot(int)
     def show_event(self, event):
@@ -714,7 +757,7 @@ class QuickView(QtWidgets.QWidget):
             ds_feats = self.rtdc_ds.features_scalar
             ds_labels = [dclab.dfn.get_feature_label(f) for f in ds_feats]
             ds_fl = sorted(zip(ds_labels, ds_feats))
-            for cb in [self.comboBox_x, self.comboBox_y]:
+            for cb in [self.comboBox_x, self.comboBox_y, self.comboBox_z_hue]:
                 fcur = cb.currentData()
                 blocked = cb.signalsBlocked()  # remember block state
                 cb.blockSignals(True)
