@@ -1,6 +1,5 @@
 # flake8: noqa: F401
 from .bg_thread import run_async
-from .colorbar_widget import ColorBarWidget
 from .double_spin_box_nan import DoubleSpinBoxNan
 from .key_value_table_widget import KeyValueTableWidget
 from .mdi_subwindow_wo_close import MDISubWindowWOButtons
@@ -10,4 +9,5 @@ from .simple_image_view import SimpleImageView
 from .simple_plot_widget import (SimplePlotItem,
                                  SimplePlotWidget,
                                  SimpleViewBox)
+from .so_colorbaritem import ShapeOutColorBarItem
 from .wait_cursor import show_wait_cursor, ShowWaitCursor
