@@ -72,6 +72,8 @@ def test_manual_basic(qtbot):
         assert ds.config["calculation"]["emodulus lut"] == "LE-2D-FEM-19"
         assert ds.config["calculation"]["emodulus medium"] == "CellCarrierB"
         assert ds.config["calculation"]["emodulus temperature"] == 29.5
+        assert ds.config["calculation"]["emodulus viscosity model"] == \
+               "buyukurganci-2022"
         assert "emodulus viscosity" not in ds.config["calculation"]
 
 
@@ -101,6 +103,8 @@ def test_manual_wrong_medium(qtbot):
         assert ds.config["calculation"]["emodulus lut"] == "LE-2D-FEM-19"
         assert ds.config["calculation"]["emodulus medium"] == "CellCarrierB"
         assert ds.config["calculation"]["emodulus temperature"] == 29.5
+        assert ds.config["calculation"]["emodulus viscosity model"] == \
+               "buyukurganci-2022"
         assert "emodulus viscosity" not in ds.config["calculation"]
 
 
@@ -126,6 +130,8 @@ def test_temperature_feature(qtbot):
         assert ds.config["calculation"]["emodulus medium"] == "CellCarrier"
         assert "emodulus temperature" not in ds.config["calculation"]
         assert "emodulus viscosity" not in ds.config["calculation"]
+        assert ds.config["calculation"]["emodulus viscosity model"] == \
+               "buyukurganci-2022"
 
 
 def test_viscosity(qtbot):
@@ -151,6 +157,7 @@ def test_viscosity(qtbot):
         assert "emodulus medium" not in ds.config["calculation"]
         assert "emodulus temperature" not in ds.config["calculation"]
         assert ds.config["calculation"]["emodulus viscosity"] == 1.0
+        assert "emodulus viscosity model" not in ds.config["calculation"]
 
 
 def test_wrong_medium_viscosity(qtbot):
@@ -178,3 +185,5 @@ def test_wrong_medium_viscosity(qtbot):
         assert ds.config["calculation"]["emodulus medium"] == "CellCarrierB"
         assert "emodulus temperature" not in ds.config["calculation"]
         assert "emodulus viscosity" not in ds.config["calculation"]
+        assert ds.config["calculation"]["emodulus viscosity model"] == \
+               "buyukurganci-2022"
