@@ -23,13 +23,15 @@ def make_fake_dataset():
         config = copy.deepcopy(ds.config)
 
     tmp = tempfile.mktemp(".rtdc", prefix="example_filter_inf_")
-    ds2 = dclab.new_dataset({"deform": np.linspace(0, .01, 100),
-                             "area_um": np.linspace(20, 200, 100),
-                             "area_ratio": np.linspace(1, 1.1, 100)
-                             })
+    ddict = {"deform": np.linspace(0, .01, 100),
+             "area_um": np.linspace(20, 200, 100),
+             "area_ratio": np.linspace(1, 1.1, 100)
+             }
+    ddict["area_ratio"][0] = np.inf
+    ddict["area_ratio"][1] = np.nan
+    ds2 = dclab.new_dataset(ddict)
     ds2.config.update(config)
-    ds2["area_ratio"][0] = np.inf
-    ds2["area_ratio"][1] = np.nan
+    ds2.config["experiment"]["event count"] = 100
     ds2.export.hdf5(tmp, features=["area_um", "deform", "area_ratio"])
     return tmp
 
@@ -136,9 +138,9 @@ def test_polygon_filter_basic(qtbot):
     with dclab.new_dataset(path) as ds:
         pf1 = dclab.PolygonFilter(
             axes=("deform", "area_um"),
-            points=[[ds["deform"].min(), ds["area_um"].min()],
-                    [ds["deform"].min(), ds["area_um"].mean()],
-                    [ds["deform"].mean(), ds["area_um"].mean()],
+            points=[[np.min(ds["deform"]), np.min(ds["area_um"])],
+                    [np.min(ds["deform"]), np.mean(ds["area_um"])],
+                    [np.mean(ds["deform"]), np.mean(ds["area_um"])],
                     ],
             name="Triangle of Death",
         )
@@ -195,9 +197,9 @@ def test_polygon_filter_delete(qtbot):
     with dclab.new_dataset(path) as ds:
         pf1 = dclab.PolygonFilter(
             axes=("deform", "area_um"),
-            points=[[ds["deform"].min(), ds["area_um"].min()],
-                    [ds["deform"].min(), ds["area_um"].mean()],
-                    [ds["deform"].mean(), ds["area_um"].mean()],
+            points=[[np.min(ds["deform"]), np.min(ds["area_um"])],
+                    [np.min(ds["deform"]), np.mean(ds["area_um"])],
+                    [np.mean(ds["deform"]), np.mean(ds["area_um"])],
                     ],
             name="Triangle of Death",
         )
